// Library:       WebSphere MQ
// Component:     Managed C++ Sample Programs
// Part:          MMQSPUT.CPP
//
// Description:   Sample Managed C++ program that puts messages to a named
//                queue.
//
//                MMQSPUT has 3 parameters:
//                - the name of a queue (required)
//                - the name of a queue manager (optional)
//                - the definition of a channel (optional)
//
//                If no queue manager name is given, the queue manager
//                defaults to the default local queue manager. If a
//                channel is defined, it should have the same format
//                as the MQSERVER environment variable.
//  <START_COPYRIGHT> 
//  Licensed Materials - Property of IBM 
//                   
//  5724-B41         
//  (C) Copyright IBM Corp. 1994, 2003 All Rights Reserved. 
//                   
//  US Government Users Restricted Rights - Use, duplication or 
//  disclosure restricted by GSA ADP Schedule Contract with 
//  IBM Corp.        
//  <END_COPYRIGHT>  

#using <mscorlib.dll>
#using <amqmdnet.dll>

#include <tchar.h>

using namespace System;
using namespace IBM::WMQ;

// This is the entry point for this application
int _tmain(int argc, char * * argv )
{
	MQQueueManager*      mqQMgr;         // MQQueueManager instance
	MQQueue*             mqQueue;        // MQQueue instance
	MQMessage*           mqMsg;          // MQMessage instance
	MQPutMessageOptions* mqPutMsgOpts;   // MQPutMessageOptions instance
	String*              queueName;      // Name of queue to use
	int                  msgLen;         // Message length
	String*              message;        // Message buffer

    System::Console::WriteLine( S"Sample MMQSPUT start" );

	if (argc < 2) 
	{
		System::Console::WriteLine( S"Required parameter missing - queue name" );
		return( (int)99 );
	}
	else 
	{
		queueName = argv[1];
	}

	///
	/// Try to create an MQQueueManager instance 
	/// 
	try 
	{
		if (argc > 3) 
		{
			// queue name, queue manager name, channel definition all provided

			// Break down the channel definition,
			// which is of the form "channel-name/transport-type/connection-name".
			String* channelDefinition = argv[3];
		    String* channelName = "";
			String* transportType = "";
			String* connectionName = "";
			Char separator[] = {'/'};
			String* parts[];
			parts = channelDefinition->Split( separator );
			if (parts.Length > 0) channelName = parts[0];
			if (parts.Length > 1) transportType = parts[1];
			if (parts.Length > 2) connectionName = parts[2];
			mqQMgr = new MQQueueManager( argv[2], channelName, connectionName );
		}
		else
			if (argc == 3) 
		{
			// queue name, queue manager name provided
			mqQMgr = new MQQueueManager( argv[2] );
		}
		else
		{
			// queue name provided - use default queue manager
			mqQMgr = new MQQueueManager();
		}
	}
	catch (MQException* mqe) 
	{
		// stop if failed
		System::Console::WriteLine( S"create of MQQueueManager with {0}", mqe->Message );
		return( (int)mqe->Reason );
	}

	///
	/// Try to open the queue
	///
	try 
	{
		mqQueue = mqQMgr->AccessQueue( queueName, 
			                           MQC::MQOO_OUTPUT                   // open queue for output
			                           + MQC::MQOO_FAIL_IF_QUIESCING );   // but not if MQM stopping
	}
	catch (MQException* mqe) 
	{
		// stop if failed
		System::Console::WriteLine( S"MQQueueManager::AccessQueue ended with {0}", mqe->Message );
		return( (int)mqe->Reason );
	}


	///
	/// Read lines from the System::Console and put them to the message queue
	/// Loop until end of input, or there is a failure
	/// 
	bool isContinue = true;
	while (isContinue) 
	{
		message = System::Console::ReadLine();
		msgLen = message->Length;

		if (msgLen > 0) 
		{
			// put the next message to the queue
			mqMsg = new MQMessage();
			mqMsg->WriteString( message );
			mqMsg->Format = MQC::MQFMT_STRING;
			mqPutMsgOpts = new MQPutMessageOptions();
			try
			{
				mqQueue->Put( mqMsg, mqPutMsgOpts );
			}
			catch (MQException* mqe)
			{
				// report the error
				System::Console::WriteLine( "MQQueue::Put ended with {0}", mqe->Message );
			}
		}
		else 
		{
			// quit loop when empty line is read
			isContinue = false;
		}
	}

	System::Console::WriteLine( "Sample MMQSPUT end" );
	return (0);
}