'* Library:       WebSphere MQ
'* Component:     MQ.NET Sample Programs
'* Part:          VMQWRLD.VB
'*
'* Description:   Sample Visual Basic program that puts and gets a message
'*                to a queue.
'*
'*                VMQWRLD has 3 parameters:
'*                - the name of a queue (optional)
'*                  e.g. SYSTEM.DEFAULT.LOCAL.QUEUE or
'*                       SYSTEM.DEFAULT.MODEL.QUEUE
'*                - the name of a queue manager (optional)
'*                - a channel definition (optional)
'*                  e.g. SYSTEM.DEF.SVRCONN/TCP/hostname(1414)
'*
'*                If no queue name is given, the name defaults to
'*                SYSTEM.DEFAULT.LOCAL.QUEUE.
'*
'*                If no queue manager name is given, the queue manager
'*                defaults to the default local queue manager.
'*  <START_COPYRIGHT> 
'*  Licensed Materials - Property of IBM 
'*                   
'*  5724-B41         
'*  (C) Copyright IBM Corp. 1994, 2003 All Rights Reserved. 
'*                   
'*  US Government Users Restricted Rights - Use, duplication or 
'*  disclosure restricted by GSA ADP Schedule Contract with 
'*  IBM Corp.        
'*  <END_COPYRIGHT> 

Imports System
Imports IBM.WMQ

Module vmqwrld

    Function Main(ByVal CmdArgs() As String) As Integer

        Const EXISTING_QUEUE As String = "SYSTEM.DEFAULT.LOCAL.QUEUE"
        Const MESSAGE As String = "Hello world"

        Dim mqQMgr As MQQueueManager                '* MQQueueManager instance
        Dim mqQueue As MQQueue                      '* MQQueue instance
        Dim queueName As String                     '* Name of queue to use
        Dim mqMsg As MQMessage                      '* MQMessage instance
        Dim mqPutMsgOpts As MQPutMessageOptions     '* MQPutMessageOptions instance
        Dim mqGetMsgOpts As MQGetMessageOptions     '* MQGetMessageOptions instance

        System.Console.WriteLine("Sample VMQWRLD start")

        '*
        '* Try to create an MQQueueManager instance 
        '* 
        Try
            If (CmdArgs.Length > 2) Then
                '* queue name, queue manager name, channel definition all provided

                '* Break down the channel definition,
                '* which is of the form "channel-name/transport-type/connection-name".
                Dim channelDefinition As String = CmdArgs.GetValue(2)
                Dim channelName As String
                Dim transportType As String
                Dim connectionName As String
                Dim separator As Char() = "/"
                Dim parts As String()

                parts = channelDefinition.Split(separator)
                If (parts.Length > 0) Then
                    channelName = parts(0)
                End If
                If (parts.Length > 1) Then
                    transportType = parts(1)
                End If
                If (parts.Length > 2) Then
                    connectionName = parts(2)
                End If
                mqQMgr = New MQQueueManager(CmdArgs.GetValue(1), channelName, connectionName)
            Else
                If (CmdArgs.Length = 2) Then
                    '* queue name, queue manager name provided
                    mqQMgr = New MQQueueManager(CmdArgs.GetValue(1))
                Else
                    '* queue name provided - use default queue manager
                    mqQMgr = New MQQueueManager()
                End If
            End If
        Catch mqe As IBM.WMQ.MQException
            '* stop if failed
            System.Console.WriteLine("create of MQQueueManager ended with {0}", mqe.Message)
            Return (mqe.Reason)
        End Try

        '*
        '* Try to open the queue
        '*
        Try
            '* If queue name provided, then use it
            '* otherwise use the default queue name
            If (CmdArgs.Length > 0) Then
                queueName = CmdArgs.GetValue(0)
            Else
                queueName = EXISTING_QUEUE
            End If
            '* The named queue may be a model queue, which will result in the
            '* creation of a temporary dynamic queue, which will be destroyed as
            '* soon as it is closed. Therefore we must ensure that such a queue
            '* is not automatically closed and re-opened. We do this by setting
            '* open options which will avoid the need for closure and re-opening.
            mqQueue = mqQMgr.AccessQueue(queueName, MQC.MQOO_OUTPUT Or MQC.MQOO_INPUT_SHARED Or MQC.MQOO_INQUIRE)
        Catch mqe As IBM.WMQ.MQException
            '* stop if failed
            System.Console.WriteLine("MQQueueManager::AccessQueue ended with {0}", mqe.Message)
            Return (mqe.Reason)
        End Try

        '*
        '* Prepare a message containing the text "Hello world".
        '* 
        mqMsg = New MQMessage()
        mqMsg.WriteString(MESSAGE)
        mqMsg.Format = MQC.MQFMT_STRING
        mqPutMsgOpts = New MQPutMessageOptions()

        '*
        '* Place the message on the queue, using default put message options.
        '* 
        Try
            mqQueue.Put(mqMsg, mqPutMsgOpts)
        Catch mqe As IBM.WMQ.MQException
            '* report the error
            System.Console.WriteLine("MQQueue::Put ended with {0}", mqe.Message)
        End Try

        '*
        '* Discover the queue manager's name.
        '* 
        Dim name As String = mqQMgr.Name
        If (name <> Nothing) Then
            System.Console.WriteLine("The queue manager name is '{0}'.", name)
        End If

        '*
        '* Show the name of the queue.
        '* 
        name = mqQueue.Name
        If (name <> Nothing) Then
            System.Console.WriteLine("Message sent to '{0}'.", name)
        End If


        '*
        '* Retrieve the data message just sent ("Hello world" expected) from the
        '* queue, using default get message options. We will get the message just
        '* sent, rather than any other message on the queue, because the "put" 
        '* will have set the id of the message in the message object, so, as we 
        '* are using the same message object, the message id acts as a filter 
        '* which says that we are only interested in a message if it has this 
        '* particular id.
        '* 
        mqGetMsgOpts = New MQGetMessageOptions()
        Try
            mqQueue.Get(mqMsg, mqGetMsgOpts)

            '* Show the text of the received message.
            System.Console.WriteLine("Message of length {0} bytes received, ", mqMsg.MessageLength)

            If (mqMsg.Format.CompareTo(MQC.MQFMT_STRING) = 0) Then
                System.Console.WriteLine(mqMsg.ReadString(mqMsg.MessageLength))
            Else
                System.Console.WriteLine("Non-text message")
            End If
            System.Console.WriteLine("Message got from queue")
        Catch mqe As IBM.WMQ.MQException
            '* report the error
            System.Console.WriteLine("MQQueue::Get ended with {0}", mqe.Message)
        End Try

        System.Console.WriteLine("Sample VMQWRLD end")
        Return (0)

    End Function

End Module
